//
//  CanvasInfo.js
//  Kite
//
//  Created by Joe Rhodes
//  Copyright (c) 2015 Joe Rhodes. All rights reserved.
//


@import 'Layer.js'
@import 'Helpers.js'


CanvasInfo = function(assetManager, sketchDoc, docURL, page, exportInfo, canvasID) {

    // Will hold a copy of the exported page
    // so we don't mutate the document
    //
    this.pageCopy = null

    this.page = page
    this.docURL = docURL
    this.sketchDoc = sketchDoc
    this.assetManager = assetManager
    this.canvasID = canvasID
    this.shapeContainerLayer = this.rootLayerForDocument(exportInfo)

    this.canvasSize = this.shapeContainerLayer.bounds.size
}

CanvasInfo.prototype = {

    canvasIdentifier: function() {
        return "Canvas-" + this.canvasID
    },

    save: function(exportInfo) {

        log("CanvasInfo Save!")
        var wasSuccessful = false

        try {
            // Create the Data/ directory if necessary
            //
            this.createDataDirectoryIfNecessary()

            // Add a bit more info to the
            // exportInfo object
            //
            exportInfo.sketchPage = this.pageCopy

            // Tell the layers to make any need preparations before
            // saving like turning of masks, etc.
            //
            this.shapeContainerLayer.prepareForSave()

            // Call save() on the hierarchy to export assets / etc.
            //
            this.shapeContainerLayer.save(exportInfo)

            // Save the layer archive to file
            //
            var canvasArchive = this.shapeContainerLayer.archivableRepresentation()
            var canvasDataFileURL = this.canvasDataFileURL()

            wasSuccessful = Helpers.writeArchiveToDestination(canvasArchive, canvasDataFileURL)
        }

        catch(error) {
            wasSuccessful = false
            Helpers.logError(error)
        }

        finally {
            // Cleanup and remove the temporary page if it exists
            //
            if (this.pageCopy != null)
            {
                var pageCopy = this.pageCopy
                var pageRemover = this.sketchDoc

                if (this.sketchDoc.documentData && this.sketchDoc.documentData().removePage) {
                    pageRemover = this.sketchDoc.documentData();
                }

                pageRemover.removePage(pageCopy)
                this.pageCopy = null
            }

            // Remove for other CanvasInfos
            //
            exportInfo.sketchPage = null
        }

        return wasSuccessful
    },

    // -------------------------------------------------------------------------------------------
    // Archiving
    // -------------------------------------------------------------------------------------------

    archivableRepresentation: function() {

        var arch = {}

        arch.canvasID = this.canvasID
        arch.canvasSize = Helpers.stringFromSize(this.canvasSize)
        arch.pointsPerSecond = 200

        return arch
    },

    // -------------------------------------------------------------------------------------------
    // Helper
    // -------------------------------------------------------------------------------------------

    dataDirectoryURL: function () {
        return this.docURL.URLByAppendingPathComponent("Data")
    },

    canvasDataFileURL: function () {
        return this.dataDirectoryURL().URLByAppendingPathComponent(this.canvasIdentifier())
    },

    createDataDirectoryIfNecessary: function () {

        var dataDirectoryURL = this.dataDirectoryURL()
        var fileManager = NSFileManager.defaultManager()

        // Does it already exist ?
        //
        if (fileManager.fileExistsAtPath( dataDirectoryURL.path() )) {
            return true
        }

        var error = null
        var successful = [fileManager createDirectoryAtURL:dataDirectoryURL withIntermediateDirectories:true attributes:null error:error]

        if (successful == false)
        {
            log("Failed to create the Data/ directory at path!! : " + dataDirectoryURL.path())
            throw error.localizedDescription()
        }

        return successful
    },

    rootLayerForDocument: function(exportInfo) {

        // Don't use 'page.immutableModelObject().newMutableCounterpart()' or
        // else detaching symbols will detach them from the origial page!
        //
        var pageCopy = this.page.copy()

        pageCopy.setName(this.page.name() + " Copy")

        this.sketchDoc.documentData().addPage(pageCopy)
        this.pageCopy = pageCopy

        var layers = pageCopy.layers()
        var artboardCount = pageCopy.artboards().count()
        var rootLayer = null

        if (artboardCount == 1 && layers.count() == 1)
        {
            log("Using Artboard for root layer")

            var artboard = pageCopy.artboards().objectAtIndex(0)
            rootLayer = new ArtboardLayer(artboard, exportInfo)

            // Single artboard documents default to the origin 0,0
            //
            rootLayer.isRootDocumentLayer = true
        }
        else {
            log("Using Page for root layer")
            rootLayer = new PageLayer(pageCopy, exportInfo)
        }

        // Create the ShapeContainerLayer that will hold the root item
        //
        if (rootLayer != null)
        {
            var canvasSize = {
                width: rootLayer.frame().width(),
                height: rootLayer.frame().height()
            }

            var containerLayer = new ShapeContainerLayer(canvasSize, exportInfo)
            containerLayer.canvasInfo = this
            containerLayer.addSublayer(rootLayer)

            return containerLayer
        }
        else {
            log("What??? Somehow we don't have a root layer?? Fix!")
            return null
        }
    }
}