//
//  DocumentInfo.js
//  Kite
//
//  Created by Joe Rhodes
//  Copyright (c) 2015 Joe Rhodes. All rights reserved.
//


@import 'CanvasInfo.js'
@import 'AssetManager.js'
@import 'Helpers.js'


DocumentInfo = function(docURL, sketchDoc, exportInfo) {

    this.docURL = docURL
    this.uuid = [[NSUUID UUID] UUIDString]
    this.creationDate = NSDate.date()
    this.assetManager = new AssetManager(docURL, sketchDoc)
    this.canvasInfos = this.canvasInfosForPagesInSketchDoc(sketchDoc, docURL, exportInfo)
    this.sketchDoc = sketchDoc

    // Save the page that was visible when we started
    // the export so we can restore it after remove
    // our page copies from the document
    //
    this.originalVisiblePage = null
}

DocumentInfo.prototype = {

    save: function (exportInfo) {

        log("DocumentInfo Save!")

        // Note down the visible page so we can restore it
        //
        this.originalVisiblePage = this.sketchDoc.currentPage()

        // Add a bit more information to the
        // exportInfo object
        //
        exportInfo.assetManager = this.assetManager

        // Save Asset Manager
        //
        if (this.assetManager.save(exportInfo) == false) {
            return false
        }

        // Save CanvasInfos
        //
        for (var canvasInfo of this.canvasInfos)
        {
            if (canvasInfo.save(exportInfo) == false) {
                return false
            }
        }

        // Restore the original visible Sketch page
        // if there was one
        //
        if (this.originalVisiblePage != null)
        {
            this.sketchDoc.setCurrentPage(this.originalVisiblePage)
            this.originalVisiblePage = null
        }

        // Write archive data to file
        //
        var archivedRep = this.archivableRepresentation()
        var documentInfoFileURL = this.documentInfoFileURL()

        return Helpers.writeArchiveToDestination(archivedRep, documentInfoFileURL)
    },

    // -------------------------------------------------------------------------------------------
    // Archiving
    // -------------------------------------------------------------------------------------------

    archivableRepresentation: function() {

        var arch = {}

        arch.uuid = this.uuid
        arch.creationDate = Helpers.stringFromNSDate(this.creationDate)
        arch.canvasInfos = this.archivedCanvasInfos()

        return arch
    },

    archivedCanvasInfos: function () {

        var canvasInfos = []

        for (var canvasInfo of this.canvasInfos) {

            var archivedCanvasInfo = canvasInfo.archivableRepresentation()

            if (archivedCanvasInfo) {
                canvasInfos.push(archivedCanvasInfo)
            }
        }

        return canvasInfos
    },

    // -------------------------------------------------------------------------------------------
    // Helper
    // -------------------------------------------------------------------------------------------

    canvasInfosForPagesInSketchDoc: function (sketchDoc, docURL, exportInfo) {

        var canvasInfos = []
        var pages = this.pagesForSketchDoc(sketchDoc, exportInfo)

        for (var pageIndex = 0; pageIndex < pages.count(); pageIndex++)
        {
            var page = pages.objectAtIndex(pageIndex)
            var canvasID = pageIndex + 1
            var canvasInfo = new CanvasInfo(this.assetManager, sketchDoc, docURL, page, exportInfo, canvasID)

            canvasInfos.push(canvasInfo)
        }

        return canvasInfos
    },

    pagesForSketchDoc: function (sketchDoc, exportInfo) {

        var pages = NSMutableArray.array()

        if (exportInfo.exportOptions.exportCurrentPageOnly)
        {
            var currentPage = this.currentPageForSketchDoc(sketchDoc)

            if (currentPage) {

                pages.addObject(currentPage)
            }
        }
        else {
            // Copy the pages into a new NSMutableArray so we don't end up
            // in an infinite loop when 'new CanvasInfo()' adds a page copy
            // to the document!
            //
            pages.addObjectsFromArray(sketchDoc.pages())
        }

        return pages
    },

    currentPageForSketchDoc: function (sketchDoc) {

        var currentPage = sketchDoc.currentPage()
        var currentPageIsMissing = (currentPage == undefined || currentPage == null)

        if (currentPageIsMissing && sketchDoc.pages().count() > 0 ) {

            currentPage = sketchDoc.pages().objectAtIndex(0)
        }

        return currentPage
    },

    documentInfoFileURL: function () {
        return this.docURL.URLByAppendingPathComponent("Info.json")
    }
}